#!/usr/bin/env python
#
#   SCOREM
#
"""
test_scorem
----------

Tests for the `scoremipsum` module.
"""
import json
import unittest
from unittest import SkipTest

import scoremipsum
from scoremipsum import game, data, util
from scoremipsum.util.conversion import convert_game_result_to_json
from scoremipsum.util.support import is_valid_json


class TestScorem(unittest.TestCase):
    """
        Test Cases for Scorem module
    """

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_game_teams_default(self):
        """

        :return:
        """
        self.assertEqual(game.TEAMS_DEFAULT,
                         ['Advancers', 'Battlers', 'Clashers', 'Destroyers',
                          'Engineers', 'Fighters', 'Guardians', 'Harriers'])

    def test_game_teams_nfl_afc_east(self):
        """

        :return:
        """
        self.assertEqual(data.TEAMS_NFL_AFC_EAST,
                         ['Patriots', 'Bills', 'Dolphins', 'Jets'])

    def test_game_team_name(self):
        """

        :return:
        """

    def test_game_get_team_default_values(self):
        """

        :return:
        """
        team_data = game.get_team_data()
        self.assertEqual(team_data, {'Offense': 2, 'Defense': 2, 'Special': 2})

    def test_game_get_score_anyball(self):
        """
            simulated result_score for single game of anyball (imaginary)

        :return:
        """
        # return 2 ints, range 0-99
        score = game.generate_score_anyball()
        assert 100 > score[0] >= 0
        assert 100 > score[1] >= 0
        print(f"\nresult_score = {score}")

    def test_game_get_score_football(self):
        """
            simulated result_score for single game of football
            nfl record for single team result_score is 73
            nfl record for both teams combined result_score is 113
        :return:
        """
        # return 2 ints, range 0-74, total < 120
        # this will be weighted for realism and tests adjusted
        score = game.generate_score_football()
        assert 75 > score[0] >= 0
        assert 75 > score[1] >= 0
        assert 120 > (score[0] + score[1]) >= 0
        print(f"\nresult_score = {score}")

    def test_game_get_score_hockey(self):
        """
            simulated result_score for single game of hockey
            nhl record for single team result_score is 16
            nhl record for both teams combined result_score is 21
        :return:
        """
        # return 2 ints, range 0-16, total < 22
        # this will be weighted for realism and tests adjusted
        score = game.generate_score_hockey()
        assert 17 > score[0] >= 0
        assert 17 > score[1] >= 0
        assert 22 > (score[0] + score[1]) >= 0
        print(f"\nresult_score = {score}")

    @SkipTest
    # test invalid until delivery of US111: SCOREM - Specify and Enforce "Away - Home" in Schedule
    def test_generate_schedule_single_pairs(self):
        schedule_set = ('always_team_AWAY', 'always_team_HOME')
        schedule = game.generate_schedule_single_pairs(schedule_set)
        assert schedule[0][0] == 'always_team_AWAY'
        assert schedule[0][1] == 'always_team_HOME'

    def test_generate_games_from_schedule(self):
        schedule_set = ('always_team_AWAY', 'always_team_HOME')
        schedule = game.generate_schedule_single_pairs(schedule_set)
        game_results = \
            game.generate_games_from_schedule(schedule, gametype='anyball')
        assert game_results is not None

    @SkipTest
    # fix imports to get this working
    def test_get_supported_sports_from_root(self):
        sports_list = scoremipsum.sports()
        self.assertEqual(sports_list, ['anyball', 'football', 'hockey'])

    def test_get_supported_sports_from_util(self):
        sports_list = scoremipsum.util.support.get_supported_sports()
        self.assertEqual(sports_list, ['anyball', 'football', 'hockey'])

    def test_is_supported_anyball(self):
        self.assertEqual(True, scoremipsum.util.support.check_support_anyball(), "Anyball not supported")

    @SkipTest
    def test_is_supported_baseball(self):
        self.assertEqual(True, scoremipsum.util.support.check_support_baseball(), "Baseball not supported")

    @SkipTest
    def test_is_supported_basketball(self):
        self.assertEqual(True, scoremipsum.util.support.check_support_basketball(), "Basketball not supported")

    def test_is_supported_football(self):
        self.assertEqual(True, scoremipsum.util.support.check_support_football(), "Football not supported")

    def test_is_supported_hockey(self):
        self.assertEqual(True, scoremipsum.util.support.check_support_hockey(), "Hockey not supported")

    def test_result_single_anyball(self):
        """
        verify results from anyball
        :return:
        """
        # schedule_set = ('Anyball_Away', 'Anyball_Home')
        schedule_set = ('Anyball_Team_AA', 'Anyball_Team_BB')
        schedule = game.generate_schedule_single_pairs(schedule_set)
        game_generation_results = \
            game.generate_games_from_schedule(schedule, gametype='anyball')
        self.assertEqual(len(schedule_set) / 2, len(game_generation_results))
        # print(f"{game_generation_results = }")

        # verify US96: Results reduce ties.  Temporary until ties are permitted.
        self.assertFalse(game_generation_results[0][0][1] == game_generation_results[0][1][1])

        game_results_json = convert_game_result_to_json(game_generation_results, gametype='anyball')
        print(f"{game_results_json = }")

        is_good_json = is_valid_json(game_results_json)
        self.assertTrue(is_good_json)
        # NOT GOOD ENOUGH FOR JSON CONTENT CHECKS THOUGH!

        gametype = json.loads(game_results_json)[0]["gametype"]
        self.assertEqual(gametype, "anyball")


    def test_result_single_football(self):
        """

        :return:
        """
        # schedule_set = ('Football_Away', 'Football_Home')
        schedule_set = ('Football_Team_AA', 'Football_Team_BB')
        schedule = game.generate_schedule_single_pairs(schedule_set)
        game_generation_results = \
            game.generate_games_from_schedule(schedule, gametype='football')
        self.assertEqual(len(schedule_set) / 2, len(game_generation_results))
        # print(f"{game_generation_results = }")

        # verify US96: Results reduce ties.  Temporary until ties are permitted.
        self.assertFalse(game_generation_results[0][0][1] == game_generation_results[0][1][1])

        game_results_json = convert_game_result_to_json(game_generation_results, gametype='football')
        print(f"{game_results_json = }")

        gametype = json.loads(game_results_json)[0]["gametype"]
        self.assertEqual(gametype, "football")

    def test_result_single_hockey(self):
        """

        :return:
        """
        # schedule_set = ('Hockey_Away', 'Hockey_Home')
        schedule_set = ('Hockey_Team_AA', 'Hockey_Team_BB')
        schedule = game.generate_schedule_single_pairs(schedule_set)
        game_generation_results = \
            game.generate_games_from_schedule(schedule, gametype='hockey')
        self.assertEqual(len(schedule_set) / 2, len(game_generation_results))
        # print(f"{game_generation_results = }")

        # verify US96: Results reduce ties.  Temporary until ties are permitted.
        self.assertFalse(game_generation_results[0][0][1] == game_generation_results[0][1][1])

        game_results_json = convert_game_result_to_json(game_generation_results, gametype='hockey')
        print(f"{game_results_json = }")

        gametype = json.loads(game_results_json)[0]["gametype"]
        self.assertEqual(gametype, "hockey")

    def test_result_multiple_anyball(self):
        """
        verify results from anyball
        :return:
        """
        schedule_set = ('AA', 'BB', 'CC', 'DD')
        schedule = game.generate_schedule_single_pairs(schedule_set)
        game_generation_results = \
            game.generate_games_from_schedule(schedule, gametype='anyball')
        self.assertEqual(len(schedule_set) / 2, len(game_generation_results))
        # print(f"{game_generation_results = }")

        multi_game_results_json = convert_game_result_to_json(game_generation_results, gametype='anyball')
        print(f"{multi_game_results_json = }")

        gametype = json.loads(multi_game_results_json)[0]["gametype"]
        self.assertEqual(gametype, "anyball")

    def test_result_multiple_football(self):
        """

        :return:
        """
        schedule_set = data.TEAMS_NFL_AFC_EAST
        schedule = game.generate_schedule_single_pairs(schedule_set)
        game_generation_results = \
            game.generate_games_from_schedule(schedule, gametype='football')
        self.assertEqual(len(schedule_set) / 2, len(game_generation_results))
        # print(f"{game_generation_results = }")

        multi_game_results_json = convert_game_result_to_json(game_generation_results, gametype='football')
        print(f"{multi_game_results_json = }")

        gametype = json.loads(multi_game_results_json)[0]["gametype"]
        self.assertEqual(gametype, "football")

    def test_result_multiple_hockey(self):
        """

        :return:
        """
        schedule_set = data.TEAMS_NHL_EASTERN_ATLANTIC
        schedule = game.generate_schedule_single_pairs(schedule_set)
        game_generation_results = \
            game.generate_games_from_schedule(schedule, gametype='hockey')
        self.assertEqual(len(schedule_set) / 2, len(game_generation_results))
        # print(f"{game_generation_results = }")

        multi_game_results_json = convert_game_result_to_json(game_generation_results, gametype='hockey')
        print(f"{multi_game_results_json = }")

        gametype = json.loads(multi_game_results_json)[0]["gametype"]
        self.assertEqual(gametype, "hockey")

    def test_schedule_all_pairs(self):
        """

        :return:
        """
        schedule_set = ('AA', 'BB', 'CC', 'DD')
        schedule = game.generate_schedule_all_pairs(schedule_set)
        schedule_expected = \
            [('AA', 'BB'), ('AA', 'CC'), ('AA', 'DD'),
             ('BB', 'CC'), ('BB', 'DD'), ('CC', 'DD')]
        self.assertEqual(schedule, schedule_expected)
        print(f"\nschedule = {schedule}")

    def test_schedule_single_pairs(self):
        """

        :return:
        """
        schedule_set = ('AA', 'BB', 'CC', 'DD')
        schedule = game.generate_schedule_single_pairs(schedule_set)
        self.assertEqual(len(sorted(schedule)), 2)
        print(f"\nschedule = {schedule}")

    def test_schedule_single_pairs_default(self):
        """

        :return:
        """
        schedule_set = game.TEAMS_DEFAULT
        schedule = game.generate_schedule_single_pairs(schedule_set)
        self.assertEqual(len(sorted(schedule)), 4)
        print(f"\ndefault teams schedule = {schedule}")

    def test_schedule_single_pairs_nfl_afc_east(self):
        """

        :return:
        """
        schedule_set = data.TEAMS_NFL_AFC_EAST
        schedule = game.generate_schedule_single_pairs(schedule_set)
        self.assertEqual(len(sorted(schedule)), 2)
        print(f"\nnfl afc east schedule = {schedule}")

    def test_schedule_single_pairs_nhl_eastern_atlantic(self):
        """

        :return:
        """
        schedule_set = data.TEAMS_NHL_EASTERN_ATLANTIC
        schedule = game.generate_schedule_single_pairs(schedule_set)
        self.assertEqual(4, len(sorted(schedule)))
        print(f"\nnhl eastern atlantic schedule = {schedule}")


if __name__ == '__main__':
    import sys

    # noinspection PyTypeChecker
    sys.exit(unittest.main())



#
#   SCOREM
#
"""
data
----------

Data for the `scoremipsum` module.
"""
TEAMS_DEFAULT = ("Advancers Battlers Clashers Destroyers "
                 "Engineers Fighters Guardians Harriers").split()

TEAMS_MLB_AL_EAST = "Yankees Orioles Red_Sox Rays Blue_Jays".split()
TEAMS_MLB_AL_CENTRAL = "".split()
TEAMS_MLB_AL_WEST = "".split()
TEAMS_MLB_NL_EAST = "".split()
TEAMS_MLB_NL_CENTRAL = "".split()
TEAMS_MLB_NL_WEST = "".split()

TEAMS_NBA_EASTERN_A = "Knicks Magic 76ers Nets Hornets".split()
TEAMS_NBA_EASTERN_B = "".split()
TEAMS_NBA_WESTERN_C = "".split()
TEAMS_NBA_WESTERN_A = "".split()
TEAMS_NBA_WESTERN_B = "".split()
TEAMS_NBA_WESTERN_C = "".split()

TEAMS_NFL_AFC_EAST = "Patriots Bills Dolphins Jets".split()
TEAMS_NFL_AFC_NORTH = "Ravens Steelers Browns Bengals".split()
TEAMS_NFL_AFC_SOUTH = "Texans Titans Colts Jaguars".split()
TEAMS_NFL_AFC_WEST = "Chiefs Raiders Broncos Chargers".split()

TEAMS_NFL_NFC_EAST = "Eagles Cowboys Giants Redskins".split()
TEAMS_NFL_NFC_NORTH = "Packers Vikings Bears Lions".split()
TEAMS_NFL_NFC_SOUTH = "Saints Falcons Buccaneers Panthers".split()
TEAMS_NFL_NFC_WEST = "49ers Seahawks Rams Cardinals".split()

TEAMS_NHL_EASTERN_ATLANTIC =\
    "Panthers Maple_Leafs Lightning Senators Red_Wings Bruins Canadiens Sabres".split()
TEAMS_NHL_EASTERN_METRO =\
    "Capitals Hurricanes Devils Blue_Jackets Rangers Islanders Flyers Penguins".split()
TEAMS_NHL_WESTERN_CENTRAL =\
    "Jets Stars Wild Avalanche Hockey_Club Blues Predators Blackhawks".split()
TEAMS_NHL_WESTERN_PACIFIC =\
    "Golden_Knights Oilers Kings Canucks Flames Ducks Kraken Sharks".split()

#   Copyright information below
#   ---
#   MLB team names are registered trademarks
#   of Major League Baseball and respective teams.
#   ---
#   NBA team names are registered trademarks
#   of the National Basketball Association and respective teams.
#   ---
#   NFL team names are registered trademarks
#   of the National Football League and respective teams.
#   ---
#   NHL team names are registered trademarks
#   of the National Hockey League and respective teams.
#   ---
#   Names of existing teams used for demonstration purposes only
#   and may not be used for any other purpose without appropriate permissions.
#   ---



#
#   SCOREM
#
"""
game
----------

game functions for the `scoremipsum` module.
"""

import random
import itertools

from scoremipsum.data import TEAMS_DEFAULT
from scoremipsum.util.scheduler import grouper


def compute_score_anyball():
    score_list = [0, 1, 2, 3, 5, 8, 13, 21]
    score = (random.choice(score_list) + random.choice(score_list)
             + random.choice(score_list) + random.choice(score_list))
    return score


def compute_score_football():
    score_list = [0, 3, 7, 10]
    score = (random.choice(score_list) + random.choice(score_list)
             + random.choice(score_list) + random.choice(score_list))
    return score


def compute_score_hockey():
    score_list = [0, 1, 2]
    score = random.choice(score_list) + random.choice(score_list) + random.choice(score_list)
    return score


def get_team_data(team=None):
    """

    :param team:    identifier for specific team.
                    must also differentiate for specific sport.
    :return:
    """
    team_data = {'Offense': 2, 'Defense': 2, 'Special': 2}
    return team_data


def generate_games_from_schedule(schedule, gametype=None):
    """
    given a schedule and game type
    return a list of game_results with scores

    - needs to implement new schedule data (home / away)

    :param schedule:
    :param gametype:
    :return:
    """
    game_results = []

    print('\ngenerating game results for: ', gametype)
    for game in schedule:
        # print('game: ', game)
        # result_score = generate_score_anyball()

        # *NOTE*
        # match command available at python version 3.10
        # -----------------------------------------------------------------------------
        # match gametype:
        #     case 'hockey':
        #         result_score = generate_score_hockey()
        #     case 'football':
        #         result_score = generate_score_football()
        #     case 'anyball':
        #         result_score = generate_score_anyball()
        #     case _:
        #         result_score = generate_score_anyball()

        #   using anyball as default
        if gametype is None:
            score = generate_score_anyball()
        elif gametype == 'hockey':
            score = generate_score_hockey()
        elif gametype == 'football':
            score = generate_score_football()
        elif gametype == 'anyball':
            score = generate_score_anyball()
        else:
            score = generate_score_anyball()

        # print('-- result_score:', result_score)
        # for team_score in result_score:
        #     print(f'-- {team_score = }:', )

        game_results.append(list(zip(game, score)))

    return game_results


def generate_schedule_all_pairs(teamlist):
    """

    :param teamlist:
    :return:
    """
    # if not teamlist:
    #     return something-hardcoded
    # generate all non-repeating pairs - placeholder algorithm
    pairs = list(itertools.combinations(teamlist, 2))

    # randomly shuffle these pairs
    #   also, change test to account for shuffling
    # random.shuffle(pairs)
    return pairs


def generate_schedule_single_pairs(teamlist):
    """

    :param teamlist:
    :return:
    """
    # if not teamlist:
    #     return something-hardcoded
    # generate first non-repeating pairs - placeholder algorithm
    tmp = list(teamlist)
    randoms = [tmp.pop(random.randrange(len(tmp))) for _ in range(len(teamlist))]
    # print('result: ', randoms)
    pairs = list(grouper(randoms, 2))
    # print('pairs: ', pairs)
    return pairs


def generate_score_anyball(ruleset=None, active_team=None, opposing_team=None):
    """
    return a result_score for Anyball (pseudogame)
    teams are rated 1-5 for Offense / Defense / Special, default 2
    result_score generation:
        actSpecAdj = actSPEC*(0-1)
        oppSpecAdj = oppSPEC*(0-1)
        actScore = actOFF*(0-2) - oppDEF*(0-2) + actSpecAdj, min 0
        oppScore = oppOFF*(0-2) - actDEF*(0-2) + oppSpecAdj, min 0
    :param ruleset:
    :param active_team:
    :param opposing_team:
    :return:
    """
    if ruleset is None:
        pass

    # score = [99, 0]
    score_visitors = compute_score_anyball()
    score_home = compute_score_anyball()

    if score_visitors == score_home:
        score_visitors, score_home = score_adjust_tie(score_visitors, score_home, game="anyball")

    score = [score_visitors, score_home]

    return score


def generate_score_football(ruleset=None, active_team=None, opposing_team=None):
    """
    return a result_score for Football (US NFL)
    teams are rated 1-5 for Offense / Defense / Special, default 2
    result_score generation:
        TBD
    :param ruleset:
    :param active_team:
    :param opposing_team:
    :return:
    """
    if ruleset is None:
        pass

    score_visitors = compute_score_football()
    score_home = compute_score_football()

    if score_visitors == score_home:
        score_visitors, score_home = score_adjust_tie(score_visitors, score_home, game="football")

    score = [score_visitors, score_home]

    return score


def generate_score_hockey(ruleset=None, active_team=None, opposing_team=None):
    """
    return a result_score for Hockey (NHL)
    future:  teams are rated 1-5 for Offense / Defense / Special, default 2
    result_score generation:
        hockey (avg 1.0 goals per period, 3 periods) - 0, 1, 2
    :param ruleset:
    :param active_team:
    :param opposing_team:
    :return:
    """
    if ruleset is None:
        pass

    score_visitors = compute_score_hockey()
    score_home = compute_score_hockey()

    if score_visitors == score_home:
        score_visitors, score_home = score_adjust_tie(score_visitors, score_home, game="hockey")

    score = [score_visitors, score_home]

    return score


def score_adjust_tie(score_visitors, score_home, game=None):
    #  print(f"*** score_adjust_for_tie {score_visitors=} {score_home=} ***")
    tiebreak_score_list = []
    if game is None:
        # don't change result_score for unspecified game
        return score_visitors, score_home

    if game == "anyball":
        # don't change result_score for anyball game! #haha
        return score_visitors, score_home

    if game == "football":
        tiebreak_score_list = [3, 6]

    if game == "hockey":
        # only one option!
        tiebreak_score_list = [1]

    tiebreak_selector = random.randint(0, 1)
    tiebreaker_score = random.choice(tiebreak_score_list)

    if tiebreak_selector:
        score_home += tiebreaker_score
    else:
        score_visitors += tiebreaker_score

    return score_visitors, score_home


class GameGeneration:
    """
    game generation class for the `scoremipsum` module.
    """

    def __init__(self, teams=None):
        if teams:
            self._teams = teams
        else:
            self._teams = TEAMS_DEFAULT

    def _team(self):
        return random.choice(self._teams)

    @staticmethod
    def get_score_anyball(active_team_data=None, opposing_team_data=None):
        """
        :param active_team_data:
        :param opposing_team_data:
        :return:
        """
        ruleset = {'anyball'}

        if not active_team_data:
            active_team_data = get_team_data()
        if not opposing_team_data:
            opposing_team_data = get_team_data()

        score = generate_score_anyball(ruleset, active_team_data, opposing_team_data)
        return score

    @staticmethod
    def get_score_football(active_team_data=None, opposing_team_data=None):
        """
        :param active_team_data:
        :param opposing_team_data:
        :return:
        """
        ruleset = {'football'}

        if not active_team_data:
            active_team_data = get_team_data()
        if not opposing_team_data:
            opposing_team_data = get_team_data()

        score = generate_score_football(ruleset, active_team_data, opposing_team_data)
        return score

    @staticmethod
    def get_score_hockey(active_team_data=None, opposing_team_data=None):
        """
        :param active_team_data:
        :param opposing_team_data:
        :return:
        """
        ruleset = {'hockey'}

        if not active_team_data:
            active_team_data = get_team_data()
        if not opposing_team_data:
            opposing_team_data = get_team_data()

        score = generate_score_hockey(ruleset, active_team_data, opposing_team_data)
        return score



"""
   scoremipsum sanity test / main
"""
import sys
from scoremipsum import game, data, scoremipsum
from scoremipsum.util.conversion import convert_game_result_to_json


def main():
    """
    scoremipsum sanity main
    """
    print("="*80)
    print('(scoremipsum sanity) :: main ::')
    print("-"*80)

    #   display the supported sports list
    #
    scoremipsum.sportsball()

    commands = scoremipsum.commands()
    print(f"== {commands = }")
    print("-"*80)

    sports = scoremipsum.sports()
    print(f"== {sports = }")
    print("-"*80)

    # get_supported_sports() direct
    # supported_sports = util.get_supported_sports()
    # print(f"== {supported_sports = }")
    # print("-"*80)

    #   display some scores!
    #
    sample = scoremipsum.game()
    print(f"== {sample = }")
    print("-"*80)


    #   display some football scores!
    #
    sample = scoremipsum.game(gametype="football")
    print(f"== {sample = }")
    print("-"*80)


    #   display some hockey scores!
    #
    sample = scoremipsum.game(gametype="hockey")
    print(f"== {sample = }")
    print("-"*80)


    #   display some more interesting scores!
    #
    teamlist = data.TEAMS_DEFAULT
    schedule = game.generate_schedule_single_pairs(teamlist)
    game_generation_results = game.generate_games_from_schedule(schedule, gametype='anyball')
    game_results_json = convert_game_result_to_json(game_generation_results, gametype='anyball')

    print(f"== {game_results_json}")
    print("-"*80)

    teamlist = data.TEAMS_NFL_AFC_EAST
    schedule = game.generate_schedule_single_pairs(teamlist)
    game_generation_results = game.generate_games_from_schedule(schedule, gametype='football')
    game_results_json = convert_game_result_to_json(game_generation_results, gametype='football')

    print(f"== {game_results_json}")
    print("-"*80)

    teamlist = data.TEAMS_NHL_EASTERN_ATLANTIC
    schedule = game.generate_schedule_single_pairs(teamlist)
    game_generation_results = game.generate_games_from_schedule(schedule, gametype='hockey')
    game_results_json = convert_game_result_to_json(game_generation_results, gametype='hockey')

    print(f"== {game_results_json}")
    print("-"*80)

    #   display a result_score like a chyron!
    #

    #   display some scores like newspaper results!
    #

    print('(scoremipsum sanity) :: end ::')
    print("="*80)
    return 0


# ----------------------------------------
if __name__ == '__main__':
    main()
    sys.exit(0)



#
#   SCOREM
#
"""
support
----------

game support utils for the `scoremipsum` module.
"""
import json

import scoremipsum


def is_valid_json(json_string):
    try:
        result = json.loads(json_string)
        return True
    except json.JSONDecodeError:
        return False


def check_support_anyball():
    """
    check for functions necessary for anyball data
    or turn off via override flag

    definition:  supported if we have config enabled, get_score function, generate_score function
    :return:
    """
    is_supported_anyball = None
    is_config_enabled_anyball = True  # future:   local config setting
    is_present_generate_score_anyball = hasattr(scoremipsum.game, "generate_score_anyball")
    is_present_get_score_anyball = hasattr(scoremipsum.game.GameGeneration, "get_score_anyball")

    if (is_config_enabled_anyball
            and is_present_get_score_anyball and is_present_generate_score_anyball):
        is_supported_anyball = True
    else:
        is_supported_anyball = False
    return is_supported_anyball


def check_support_baseball():
    """
    check for functions necessary for baseball data
    or turn off via override flag

    definition:  supported if we have config enabled, get_score function, generate_score function
    :return:
    """
    is_supported_baseball = None
    is_config_enabled_baseball = True  # future:   local config setting
    is_present_generate_score_baseball = hasattr(scoremipsum.game, "generate_score_baseball")
    is_present_get_score_baseball = hasattr(scoremipsum.game.GameGeneration, "get_score_baseball")

    if (is_config_enabled_baseball
            and is_present_get_score_baseball and is_present_generate_score_baseball):
        is_supported_baseball = True
    else:
        is_supported_baseball = False
    return is_supported_baseball


def check_support_basketball():
    """
    check for functions necessary for basketball data
    or turn off via override flag

    definition:  supported if we have config enabled, get_score function, generate_score function
    :return:
    """
    is_supported_basketball = None
    is_config_enabled_basketball = True  # future:   local config setting
    is_present_generate_score_basketball = hasattr(scoremipsum.game, "generate_score_basketball")
    is_present_get_score_basketball = hasattr(scoremipsum.game.GameGeneration, "get_score_basketball")

    if (is_config_enabled_basketball
            and is_present_get_score_basketball and is_present_generate_score_basketball):
        is_supported_basketball = True
    else:
        is_supported_basketball = False
    return is_supported_basketball


def check_support_football():
    """
    check for functions necessary for football data
    or turn off via override flag

    definition:  supported if we have config enabled, get_score function, generate_score function
    :return:
    """
    is_supported_football = None
    is_config_enabled_football = True  # future:   local config setting
    is_present_generate_score_football = hasattr(scoremipsum.game, "generate_score_football")
    is_present_get_score_football = hasattr(scoremipsum.game.GameGeneration, "get_score_football")

    if (is_config_enabled_football
            and is_present_get_score_football and is_present_generate_score_football):
        is_supported_football = True
    else:
        is_supported_football = False
    return is_supported_football


def check_support_hockey():
    """
    check for functions necessary for hockey data
    or turn off via override flag

    definition:  supported if we have config enabled, get_score function, generate_score function
    :return:
    """
    is_supported_hockey = None
    is_config_enabled_hockey = True  # future:   local config setting
    is_present_generate_score_hockey = hasattr(scoremipsum.game, "generate_score_hockey")
    is_present_get_score_hockey = hasattr(scoremipsum.game.GameGeneration, "get_score_hockey")

    if (is_config_enabled_hockey
            and is_present_get_score_hockey and is_present_generate_score_hockey):
        is_supported_hockey = True
    else:
        is_supported_hockey = False
    return is_supported_hockey


def get_supported_sports():
    """
    list all sports for which the associated support check passes
    :return:
    """
    supported_sports = []

    if check_support_anyball():
        supported_sports.append('anyball')

    if check_support_baseball():
        supported_sports.append('baseball')

    if check_support_basketball():
        supported_sports.append('basketball')

    if check_support_football():
        supported_sports.append('football')

    if check_support_hockey():
        supported_sports.append('hockey')

    return supported_sports



#
#   SCOREM
#
"""
Scorem
----------

Scorem functions for the `scoremipsum` module.
"""
import scoremipsum
from scoremipsum.util.support import get_supported_sports


def game(gametype=None):
    print("game() not yet implemented !")


def commands():
    method_list = [func for func in dir(scoremipsum.scoremipsum) if
                   callable(getattr(scoremipsum.scoremipsum, func)) and not func.startswith(
                       "_") and not func.startswith("get_")]
    return method_list


def help():
    print("help() not yet implemented !")


def sportsball():
    print("== sportsball !")
    print("-" * 80)


def sports():
    sports_list = get_supported_sports()
    return sports_list



#
#   SCOREM
#
"""
conversion
----------

conversion utils for the `scoremipsum` module.
"""
import json


def convert_game_result_to_json(result_score, gametype=None):
    result_score_dict = []
    result_score = result_score
    gametype = gametype

    keys = ["gametype", "team_away", "score_away", "team_home", "score_home"]

    for res in result_score:
        gametype = gametype
        team_away, score_away = res[0][0], res[0][1]
        team_home, score_home = res[1][0], res[1][1]
        values = gametype, team_away, score_away, team_home, score_home
        game_dict = dict(zip(keys, values))
        result_score_dict.append(game_dict)

    # json_data = result_score_dict
    result_score_json = json.dumps(result_score_dict, indent=4)
    result_score_list = json.loads(result_score_json)
    print(f":::: conversion :: {result_score_list = }")

    return result_score_json



#
#   SCOREM
#
"""
scheduler
----------

scheduler utils for the `scoremipsum` module.
"""


def grouper(inputs, n=2):
    """

    :param inputs:
    :param n:
    :return:
    """
    iters = [iter(inputs)] * n
    return zip(*iters)
